/* FreewareUpdate */

#import <Cocoa/Cocoa.h>
#import <HMBlkAppKit/HMBlkAppKit.h>
#import "Thread.h"
#import "CGSPrivate.h"
#import "UpdatesTableView.h"


#define UPD_URL			@"http://www.macfreeware.fr/MacFreewareUpdate.plist"
#define NEWS_URL		@"http://macfreeware.fr/getNewsMFU.php?lang=en&id="

#define USE_DAEMON				@"Use_Daemon"
#define DAEMON_FREQUENCY		@"Daemon_Frequency"
#define DAEMON_FIRSTUSE			@"Daemon_First_Use"
#define DAEMON_NEXTCHECK		@"Daemon_Next_Check"
#define KEEP_UPDATES			@"Keep_Updates"
#define KEEP_FOLDER				@"Keep_Folder"
#define DEFAULT_KEEP_FOLDER		@"Default_Keep_Folder"
#define APP_TO_CHECK			@"App_To_Check"
#define APP_TO_CHECK_ACTIVE		@"Active_App_To_Check"
#define APP_NOT_TO_CHECK_ACTIVE	@"Active_No_Check_For_App"
#define ALLOWS_INSTALLATION		@"Allows_Installation"


// FREQUENCY : 0= Every Months ; 1= Every Weeks ; 2= Every Days ; 3= Every Hours

@interface FreewareUpdate : NSObject <ThreadedDelegate>
{
	// Views
	IBOutlet NSBox*		mainBox;
	IBOutlet NSView*	updatesView;
	IBOutlet NSView*	checkingView;
	IBOutlet HMBlkPanel*	window;
	IBOutlet NSView*	mainView;
	IBOutlet NSBox*		updatesBox;
	IBOutlet NSView*	updatesChoicesView;
	IBOutlet NSView*	updatesDownloadView;
	IBOutlet NSView*	errorView;
	IBOutlet NSView*	noUpdatesView;
	IBOutlet NSView*	preferencesView;
	IBOutlet NSView*	aboutView;
	IBOutlet NSView*	helpView;
	IBOutlet NSView*	finishView;
	IBOutlet NSView*	newsView;
	IBOutlet NSView*		freewareView;
	IBOutlet HMBlkTableView*	freewareTableView;
	IBOutlet HMBlkTableView*	prefsTableView;
	
	
	IBOutlet NSMenu*	statusMenu;

	IBOutlet HMBlkProgressIndicator*	progressView;
	IBOutlet UpdatesTableView*	tableView;
	IBOutlet NSTextField*	errorsField;
	
	// Controllers
	NSUserDefaults*		prefs;
	NSFileManager*		fileManager;
	Thread*				thread;
	NSMutableArray*		updatesArray;
	NSMutableArray*		freewareArray;
	NSMutableArray*		activeFreewareArray;
	NSStatusItem*		statusItem;
	NSTimer* checkTimer;
	
	// Loading
	IBOutlet NSTextField*	loadingUpdatesField;
	
	// Preferences
	IBOutlet NSButton*		daemonCheckboxButton;
	IBOutlet NSButton*		daemonPopUpButton;
	IBOutlet NSTextField*	daemonCheckTimeField;
	IBOutlet NSMenu*		daemonFrenquencyMenu;
	IBOutlet NSButton*		receiptsCheckboxButton;
	IBOutlet NSButton*		receiptsChooseButton;
	IBOutlet NSTextField*	receiptsPathField;
	IBOutlet NSButton*		allowsInstallationCheckboxButton;
	IBOutlet NSButton*		appToUpdateCheckboxButton;
	IBOutlet NSButton*		appToNotUpdateCheckboxButton;
	
	// News
	IBOutlet NSTextView*	newsTextView;
	IBOutlet NSProgressIndicator*	newsProgressIndicator;
	
	// Freewares
	IBOutlet HMBlkProgressIndicator*	freewareProgressIndicator;
	IBOutlet NSTextField*				freewareCountField;
	
	// BOOL
	BOOL workingAsDaemon; // YES = Freeware Update window is closed, but it keeps working. NO = Nothing is working.
	BOOL isCheckingUpdates; // Necessary when daemon is active. If = YES, then daemon won't check for updates and will report time.

	// INT
	int choice; // The choice for NSRunAlertSheet
}


// Checking
- (IBAction)checkForUpdates:(id)sender;
- (void)setTimeIntervalForCheck;
- (void)beginCheckForUpdates:(NSURL*)updatesURL;
- (void)unableToGetDataBase:(id)sender;
- (void)ckeckingEndedWithUpdates:(BOOL)value;

// Download
- (IBAction)download:(id)sender;

// Actions
- (IBAction)cancel:(id)sender;
- (IBAction)contact:(id)sender;
- (IBAction)showPreferences:(id)sender;
- (IBAction)showAbout:(id)sender;
- (IBAction)backToUpdates:(id)sender;
- (IBAction)showHelp:(id)sender;
- (IBAction)showFreeware:(id)sender;
- (IBAction)revealAndQuit:(id)sender;
- (IBAction)closeWindow:(id)sender;


// Preferences
- (void)initializePreferences;
- (IBAction)activeDaemon:(id)sender;
- (IBAction)setDaemonFreqency:(id)sender;
- (IBAction)setKeepReceipts:(id)sender;
- (IBAction)setReceiptsDestination:(id)sender;
- (IBAction)setAppToCheck:(id)sender;
- (IBAction)setAppToNotCheck:(id)sender;
- (IBAction)setAllowsInstallation:(id)sender;
- (void)removeToLoginItems:(NSString*)path hide:(BOOL)aValue;
- (void)insertToLoginItems:(NSString*)path hide:(BOOL)aValue;


// Animation
- (void)animateWindow:(NSWindow*)win effect:(CGSTransitionType)fx direction:(CGSTransitionOption)dir
			 duration:(float)dur;

@end
